/* **************************************************************************
3D Icon/Bitmap Button Function

Copyright (C) 1991 Mark S. Douglas
All Rights Reserved.

11/11/91   MDouglas     Created.
************************************************************************** */


#include <windows.h>
#include <3dbutton.h>


#define RGB_LTGRAY      RGB(192,192,192)
#define RGB_DARKGRAY    RGB(128,128,128)
#define RGB_MEDIUMGRAY  RGB(160,160,164)
#define RGB_WHITE       RGB(0xFF,0xFF,0xFF)


#define IsButtonSelected(lpd)   (lpd->itemAction == ODA_SELECT && lpd->itemState & ODS_SELECTED)


static int  cxIcon;
static int  cyIcon;


VOID    WndDraw3DRect(HDC hdc,int x,int y,int cx,int cy,WORD wStyle,WORD wThickness)
{
    //**********************************************************************
    //*                                                                    *
    //* Draws a 3D rectangle.                                              *
    //*                                                                    *
    //* hdc         DC to draw into.                                       *
    //* x           The logical x-coordinate of the upper left corner of   *
    //*             the rectangle.                                         *
    //* y           The logical y-coordinate of the upper left corner of   *
    //*             the rectangle.                                         *
    //* cx          The width of the rectangle.                            *
    //* cy          The heigth of the rectangle                            *
    //* wStyle      Specifies the button style - Raised (DR_UP),           *
    //*             Recessed (DR_DOWN), or Pushed (DR_PUSHED).             *
    //* wThickness  Thickness of the 3D Border.  This is ignored for when  *
    //*             the DR_PUSHED style is specified.                      *
    //*                                                                    *
    //* NOTE:	    You can also specifiy one logical pixel coordinate by  *
    //*             using the flag DR_BORDER in the wStyle paramater.      *
    //*                                                                    *
    //**********************************************************************

    POINT   apt[3];
    WORD    wIndex;
    HBRUSH  hbrOld;
    HPEN    hpenLower;
    DWORD   rgbLowerCorner;
    HPEN    hpenOld;
    HPEN    hpenUpper;
    DWORD   rgbUpperCorner;

    if(wStyle & DR_BORDER) {
        hbrOld = SelectObject(hdc,GetStockObject(HOLLOW_BRUSH));
        Rectangle(hdc,x,y,cx,cy);
        SelectObject(hdc,hbrOld);

        x ++;
        y ++;
        cx --;
        cy --;
    }

    if(wStyle & DR_DOWN || wStyle & DR_PUSHED) {
        rgbUpperCorner = RGB_DARKGRAY;
        rgbLowerCorner = RGB_WHITE;
    }
    else {
        rgbUpperCorner = RGB_WHITE;
        rgbLowerCorner = RGB_DARKGRAY;
    }

    hpenLower = CreatePen(PS_SOLID,1,rgbLowerCorner);
    hpenUpper = CreatePen(PS_SOLID,1,rgbUpperCorner);
    hpenOld   = SelectObject(hdc,hpenLower);

    cx --;
    cy --;

    for(wIndex = 0;wIndex < wThickness;wIndex ++) {
        apt[0].x = x + wIndex;
        apt[0].y = cy - wIndex;
        apt[1].x = apt[0].x;
        apt[1].y = y + wIndex;
        apt[2].x = cx + 1 - wIndex;
        apt[2].y = apt[1].y;
        SelectObject(hdc,hpenUpper);
        Polyline(hdc,apt,3);

        if(wStyle & DR_PUSHED) break;

        apt[0].x ++;
        apt[2].x --;
        apt[1].x = apt[2].x;
        apt[1].y = apt[0].y;
        SelectObject(hdc,hpenLower);
        Polyline(hdc,apt,3);
    }

    SelectObject(hdc,hpenOld);
    DeleteObject(hpenLower);
    DeleteObject(hpenUpper);
}


VOID    WndDrawBitmap(HDC hdc,HBITMAP hbm,int x,int y,int cx,int cy)
{
    //**********************************************************************
    //*                                                                    *
    //* Draws a bitmap into a DC.                                          *
    //*                                                                    *
    //* hdc     DC to draw into.                                           *
    //* hbm     Bitmap to draw.                                            *
    //* x       If x is equal to -1 the bitmap is centered within the cx   *
    //*         width.                                                     *
    //* y       If y is equal to -1 the bitmap is centered within the cy   *
    //*         height.                                                    *
    //* cx      If cx is zero the width is set to the width of the bitmap. *
    //* cy      If cy is zero the height is set to the height of the       *
    //*         bitmap.                                                    *
    //*                                                                    *
    //* NOTE:	If the width or height of the bitmap differ from cx or cy, *
    //*         the bitmap is stretched or schrunk accordingly             *
    //*         To avoid any stretching or schrinking, set cx and cy to    *
    //*         zero.                                                      *
    //*                                                                    *
    //**********************************************************************

    BITMAP  bitmap;
    HBITMAP hbmOld;
    HDC     hdcSrc;

    hdcSrc = CreateCompatibleDC(hdc);
    if(hdcSrc) {
        GetObject(hbm,sizeof(BITMAP),(LPSTR)&bitmap);

        if(!cx) cx = bitmap.bmWidth;
        if(x == -1) {
            x  = (cx - bitmap.bmWidth) / 2;
            cx = bitmap.bmWidth;
        }

        if(!cy) cy = bitmap.bmHeight;
        if(y == -1) {
            y  = (cy - bitmap.bmHeight) / 2;
            cy = bitmap.bmHeight;
        }

        hbmOld = SelectObject(hdcSrc,hbm);
        StretchBlt(hdc,x,y,cx,cy,hdcSrc,0,0,bitmap.bmWidth,bitmap.bmHeight,SRCCOPY);
        SelectObject(hdcSrc,hbmOld);

        DeleteDC (hdcSrc);
    }
}


VOID    WndDrawBitmapButton(LPDRAWITEMSTRUCT lpdrawi,HBITMAP hbm)
{
    //**********************************************************************
    //*                                                                    *
    //* Draws a 3D button and a bitmap in the center of the button.        *
    //*                                                                    *
    //* lpdrawi     A long pointer to a DRAWITEMSTRUCT received as the     *
    //*             lParam of WM_DRAWITEM message.                         *
    //* hbm         The bitmap to draw in the center of the button.        *
    //*                                                                    *
    //**********************************************************************

    BITMAP  bitmap;
    BOOL    bPushed;

    if(lpdrawi->itemAction == ODA_DRAWENTIRE || lpdrawi->itemAction == ODA_SELECT) {
        bPushed = IsButtonSelected(lpdrawi);

        FillRect(lpdrawi->hDC,&lpdrawi->rcItem,GetStockObject(LTGRAY_BRUSH));

        GetObject(hbm,sizeof(BITMAP),(LPSTR)&bitmap);
        WndDrawBitmap(lpdrawi->hDC,hbm,(lpdrawi->rcItem.right - bitmap.bmWidth) / 2 + bPushed,
                      (lpdrawi->rcItem.bottom - bitmap.bmHeight) / 2 + bPushed,bitmap.bmWidth,bitmap.bmHeight);

        WndDraw3DRect(lpdrawi->hDC,lpdrawi->rcItem.left,lpdrawi->rcItem.top,lpdrawi->rcItem.right,lpdrawi->rcItem.bottom,
                      DR_BORDER|((!bPushed) ? DR_UP : DR_PUSHED),2);
    }
}


VOID    WndDrawIconButton(LPDRAWITEMSTRUCT lpdrawi,HICON hicon)
{
    //**********************************************************************
    //*                                                                    *
    //* Draws a 3D button and an icon in the center of the button.         *
    //*                                                                    *
    //* lpdrawi  A long pointer to a DRAWITEMSTRUCT received as the lParam *
    //*          of the WM_DRAWITEM message.                               *
    //* hicon	 The icon to draw in the center of the button.	If hicon   *
    //*          is NULL this function uses the window text for the button *
    //*          as the name of the icon and loads the icon automatically. *
    //*                                                                    *
    //* NOTE:	 Setting the window text of the button to the name of an   *
    //*          icon is the easiest way to specify the icon to draw.  If  *
    //*          you use this method make sure to specify hicon as NULL    *
    //*          when you call this function.                              *
    //*                                                                    *
    //**********************************************************************

    BOOL    bPushed;
    char    szStr[32];

    if(lpdrawi->itemAction == ODA_DRAWENTIRE || lpdrawi->itemAction == ODA_SELECT) {
        if(!cxIcon) {
            cxIcon = GetSystemMetrics(SM_CXICON);
            cyIcon = GetSystemMetrics(SM_CYICON);
        }

        if(!hicon) {
            GetWindowText(lpdrawi->hwndItem,szStr,sizeof(szStr));
            hicon = LoadIcon(GetWindowWord(lpdrawi->hwndItem,GWW_HINSTANCE),szStr);
        }

        bPushed = IsButtonSelected(lpdrawi);

        FillRect(lpdrawi->hDC,&lpdrawi->rcItem,GetStockObject(LTGRAY_BRUSH));

        if(hicon) DrawIcon(lpdrawi->hDC,(lpdrawi->rcItem.right - cxIcon) / 2 + bPushed,
                           (lpdrawi->rcItem.bottom - cyIcon) / 2 + bPushed,hicon);

        WndDraw3DRect(lpdrawi->hDC,lpdrawi->rcItem.left,lpdrawi->rcItem.top,lpdrawi->rcItem.right,lpdrawi->rcItem.bottom,
                      DR_BORDER|((!bPushed) ? DR_UP : DR_PUSHED),2);
    }
}
