/////////////////////////////////////////////////////////////////////
//	Library		: NetWork Collection Kit
//	File		: NWCK32.HPP
/////////////////////////////////////////////////////////////////////
//	Author		: Ravi Prakash
//	Date		: November 1995
/////////////////////////////////////////////////////////////////////
//	Revision History
//
/////////////////////////////////////////////////////////////////////
//	Copyright 1995-96, Sapient Technology, Inc.  All Rights Reserved.
/////////////////////////////////////////////////////////////////////

#ifndef NWCK32_HPP
#define NWCK32_HPP

#include <winsock.h>
/***  FUNCTION PROTOTYPES  ***/

#ifdef __cplusplus
extern "C" {
#endif

/***********************************************************************
	void DLLMETHOD NWCKInit()

	Parameters:
		None

	Returns:
		None

	Notes:
		You must call this function first before using any other functions
		in this DLL.  If you are using the 'Evaluation Only' copy for 30
		days, then it will display a splash window.  If you fail to call this
		function first and then use another function in the DLL it will return
		-1 or NULL.

		If you have the real registered version then it does nothing.  However 
		even if you have a registered version you should still call this 
		function.  Since it could be used in the future releases to do some 
		other king of initialization.
***********************************************************************/
void DLLMETHOD NWCKInit();

/***********************************************************************
	DWORD DLLMETHOD NWCKVersion()

	Parameters:
		None

	Returns:
		HIWORD(ret) == major version
		LOWORD(ret) == major version

	Notes:
		None.
***********************************************************************/
DWORD DLLMETHOD NWCKVersion();

/***********************************************************************
	BOOL DLLMETHOD NWCKGetServerNameDlg( HWND Parent, char * Buffer )

	Parameters:
		Parent	- Parent Window handle
		Buffer	- This is a pointer to an array of atleast 32 characters

	Returns:
		TRUE	- If user CLICKED on IDOK.  Buffer will contain the server
				  name eg: "MYNTSERVER"
		FALSE	- User canceled dialog.  Data in Buffer is not valid

	Notes:
		None.
***********************************************************************/
BOOL DLLMETHOD NWCKGetServerNameDlg( HWND Parent, char * Buffer );


/*********  WINDOWS SOCKET SUPPORT   *********/

/***********************************************************************
	BOOL DLLMETHOD NWCKStartWinSock( WORD minVersion );

	Parameters:
		minVersion	- The current version as of Dec 95 is still 1.1.  You must 
					  set this to 1.1.  The LOWBYTE() is the Major Version and 
					  the HIBYTE() is the Minor version.

	Returns:
		TRUE	- If wsock32.dll is initialized ok.
		FALSE	- Failure.  Don't use any other winsock functions.
				  DO NOT CALL NWCKLastWinSockError()!

	Notes:
		You must call this function 1st before you start using Winsock.  This
		function will initialize winsock and create certain variables.
***********************************************************************/
BOOL DLLMETHOD NWCKStartWinSock( WORD minVersion );

/***********************************************************************
	BOOL DLLMETHOD NWCKEndWinSock();

	Parameters:
		None.

	Returns:
		TRUE	- ok.
		FALSE	- Failure.
				  DO NOT CALL NWCKLastWinSockError()!

	Notes:
		You must call this function when you have finished using winsock in
		your application.  This closes winsock.  After this call you will need
		to call NWCKStartWinSock() if you want to use winsock again.
***********************************************************************/
BOOL DLLMETHOD NWCKEndWinSock();

/***********************************************************************
	int DLLMETHOD NWCKGetWinSockInfo( WORD *ver, char *desc, char *stat, 
					short *maxSock, short *maxDGram, char *vendor );

	Parameters:
		ver		- The winsock DLL version. LOBYTE = major HIBYTE = minor vers.
		desc	- An array of minimum 257 bytes for the description.
		stat	- An array of minimum 129 bytes for System status.
		maxSock	- Maximum number of sockets per process
		maxDGram- Maximum size of a datagram packet in bytes.
		vendor	- An array of minimum 257 bytes for vendor specific data.

	Returns:
		0	- Success all data is filled up for you to use
		-1	- Winsock not initialized
			  DO NOT CALL NWCKLastWinSockError()!

	Notes:
		None.
***********************************************************************/
int DLLMETHOD NWCKGetWinSockInfo( WORD *ver, char *desc, char *stat, 
							short *maxSock, short *maxDGram, char *vendor );

/***********************************************************************
	int DLLMETHOD NWCKGetMyName( char *name, char *ipaddr );

	Parameters:
		name	- A pointer to an array of minimum 40 bytes.
		ipaddr	- A pointer to an array of minimum 128 bytes.

	Returns:
		0	- Success 
			  name will have the host machine name (your machine). 
			  ipaddr will have an array of dotted ip address 
			  i.e. 223.223.223.1.  Some machines may have multiple address 
			  with different network cards.  The 'ipaddr' buffer will hold 
			  a maximum of 3 ip addresses.  Each address will be separated 
			  by a '+' character.  So the resulting buffer could look like:
			  "223.223.223.1+223.223.223.2+223.223.223.3"
			  It is upto you to parse this string
		-1	- Winsock not initialized
		-2	- Winsock error ( call NWCKLastWinSockError() )

	Notes:
		None.
***********************************************************************/
int DLLMETHOD NWCKGetMyName( char *name, char *ipaddress );

/***********************************************************************
	int DLLMETHOD NWCKGetServerIP( char *name, char *ipaddr );

	Parameters:
		name	- A pointer to the machine name i.e. "JACK" (remote machine)
		ipaddr	- A pointer to an array of minimum 128 bytes.

	Returns:
		0	- Success 
			  ipaddr will have an array of dotted ip address 
			  i.e. 223.223.223.1.  Some machines may have multiple address 
			  with different network cards.  The 'ipaddr' buffer will hold 
			  a maximum of 3 ip addresses.  Each address will be separated 
			  by a '+' character.  So the resulting buffer could look like:
			  "223.223.223.1+223.223.223.2+223.223.223.3"
			  It is upto you to parse this string
		-1	- Winsock not initialized
		-2	- Winsock error ( call NWCKLastWinSockError() )

	Notes:
		None.
***********************************************************************/
int DLLMETHOD NWCKGetServerIP( char *name, char *ipaddress );

/***********************************************************************
	int DLLMETHOD NWCKLastWinSockError( int *err, char *errStr );

	Parameters:
		err		- A pointer to an 'int' for error #
		errStr	- A pointer to an array of minimum 100 bytes.  Error message

	Returns:
		0	- Success 
			  err will have an error #
			  errStr will have an error message suitable for display
		-1	- Winsock not initialized

	Notes:
		There 3 versions of 'NWCKLastWinSockError'.  This one should
		be used only when you have no socket handle.  The other two are
		for socket handles, one each for UDP and Stream sockets.  Errors
		are tracked on a per socket basis.
***********************************************************************/
int DLLMETHOD NWCKLastWinSockError( int *err, char *errStr );

/***********************************************************************
						DATAGRAM SOCKET API

All datagram socket APIs have the word 'UDP' embedded in them.  Please be
sure to understand which APIs to use if your process is a server and which
ones to use if it is a client.

SERVER
As a server you will typically call one of the NWCKCreateUDPServerX() 
functions.  Next you wait till your thread or window receives the user defined
message you set up earlier. When you receive this message the WPARAM
parameter will have a value of NWCK_UDP_RECVDATA.  The LPARAM will contain
the size of the data received in bytes.  Your next step would be to allocate
x bytes of memory and call NWCKUDPRead() function.  The WPARAM could also
be NWCK_UDP_READERROR.  This means some data was sent to us but there was an
error in receiving the data.  Call NWCKLastUDPWinSockError() to find out what 
occurred.

If you wish to send back a response or data you MUST use the NWCKUDPReply()
function.  DO NOT USE THE NWCKUDPWriteXXX() functions.  If the function is 
successful you can free the memory associated with the data since the DLL
makes a copy it.  After this call you must wait for a message sent back to 
your Window/Thread.  The message id will be the same as you set earlier.  
The WPARAM will contain the value NWCK_UDP_DATASENT.  The LPARAM is not 
used.  The WPARAM could also be NWCK_UDP_WRITEERROR, this means there was 
an error.  You must then call NWCKLastUDPWinSockError().

CLIENT
As a client you need to know who you are going to communicate with.  You need
to know the UPD Servers machine name and the Port # of the service or the 
service name itself.

First you would call one of the NWCKCreateUDPClientX() functions.  Now you
can start sending data.  To send data you must call one of the 
NWCKUDPWriteToXXX() functions.  If the function is successful you can
free the memory associated with your data since the DLL makes a copy of it.
After this function you must wait for a message sent to your Window/Thread.
The message id will be what you set up earlier.  The WPARAM will have a 
value of NWCK_UDP_DATASENT.

If the UDP Server sent you a response/data, you will receive your user 
defined message to your Window/Thread.  When you receive this message the 
WPARAM parameter will have a value of NWCK_UDP_RECVDATA.  The LPARAM will 
contain the size of the data received in bytes.  Your next step would be to 
allocate x bytes of memory and call NWCKUDPRead() function.  As a client 
process you will ignore the 'returnAddr' buffer data that is filled up for 
you by the NWCKUDPRead() function.

If you need to send data again to the Server, you will need to go through the
above steps again like you did when you first sent data.

MESSAGES
All messages will be sent to your Window in the form of PostMessage API.  
You can also receive messages to your thread in the form of 
PostThreadMessage API.  The destination is decided by you depending on 
which NWCKCreateUDPServerX() function you call.  The "long handle" parameter 
should be preserved by you, it is used later to uniquely identify which 
thread or window instance owns the socket.

When you are done using a UDP socket you MUST call NWCKCloseUDP().  This
will allow the DLL to properly close the socket and to free all memory
associated with that instance.

MULITHREADED
The DLL is thread safe and can handle multiple threads.  The DLL uses all 
the Asynchronous APIs for Winsock.  So you don't really need to have a 
separate thread to handle Winsock message.  Your application should be 
quite responsive using just the main thread.

***********************************************************************/

#define	NWCK_UDP_RECVDATA		15000
#define	NWCK_UDP_DATASENT		15001
#define	NWCK_UDP_WRITEERROR		15002
#define	NWCK_UDP_READERROR		15003
#define	NWCK_UDP_SHUTDOWN		15004

#define	NWCK_STREAM_RECVDATA	15005
#define	NWCK_STREAM_DATASENT	15006
#define	NWCK_STREAM_WRITEERROR	15007
#define	NWCK_STREAM_READERROR	15008
#define	NWCK_STREAM_CONNECT		15009
#define	NWCK_STREAM_ACCEPT		15010
#define	NWCK_STREAM_SHUTDOWN	15011

// Additional constants
#define NWCK_WINSOCK_NOERROR		0
#define NWCK_WIN32_ERROR      		1
#define NWCK_WIN32_MEM_ERROR      	2
#define NWCK_WIN32_PRG_ERROR		3
#define NWCK_STREAM_NOTSERVER		4


/***********************************************************************
 	int DLLMETHOD NWCKCreateUDPServer1( int localPort, HWND pHwnd, UINT uMesg,
				long *handle );

	Parameters:
		localPort	- An unique port #.  This will be used to bind the socket
		pHwnd		- A window handle that will receive notification of socket
					  events.
		uMesg		- A WM_USER + XXX mesage that will be sent to pHwnd
		handle		- A pointer to a long.  On success this will contain an
					  unique handle.  You need to use this handle in all 
					  subsequent calls
	Returns:
		0	- Success 
		-1	- Winsock not initialized
		-2	- Winsock error ( call NWCKLastWinSockError() )
		-3	- Bad 'localPort' #
		-4	- Low Memory problem 
			  Do NOT call NWCKLastWinSockError() since the handle is invalid

	Notes:
		None.
***********************************************************************/
int DLLMETHOD NWCKCreateUDPServer1( int localPort, HWND pHwnd, UINT uMesg,
																long *handle );

/***********************************************************************
 	int DLLMETHOD NWCKCreateUDPServer2( int localPort, DWORD pThread, UINT uMesg,
				long *handle );

	Parameters:
		localPort	- An unique port #.  This will be used to bind the socket
		pThread		- A thread id that will receive notification of socket
					  events
		uMesg		- A WM_USER + XXX mesage that will be sent to pHwnd
		handle		- A pointer to a long.  On success this will contain an
					  unique handle.  You need to use this handle in all 
					  subsequent calls
	Returns:
		0	- Success 
		-1	- Winsock not initialized
		-2	- Winsock error ( call NWCKLastWinSockError() )
		-3	- Bad 'localPort' #
		-4	- Low Memory problem 
			  Do NOT call NWCKLastWinSockError() since the handle is invalid

	Notes:
		None.
***********************************************************************/
int DLLMETHOD NWCKCreateUDPServer2( int localPort, DWORD pThread, UINT uMesg,
																long *handle );

/***********************************************************************
 	int DLLMETHOD NWCKCreateUDPClient1( HWND pHwnd, UINT uMesg, long *handle );

	Parameters:
		pHwnd		- A window handle that will receive notification of socket
					  events.
		uMesg		- A WM_USER + XXX mesage that will be sent to pHwnd
		handle		- A pointer to a long.  On success this will contain an
					  unique handle.  You need to use this handle in all 
					  subsequent calls
	Returns:
		0	- Success 
			  ipaddr will have a single dotted ip address i.e. 223.223.223.1
		-1	- Winsock not initialized
		-2	- Winsock error ( call NWCKLastWinSockError() )
		-3	- Not used (ignore)
		-4	- Low Memory problem 
			  Do NOT call NWCKLastWinSockError() since the handle is invalid

	Notes:
		None.
***********************************************************************/
int DLLMETHOD NWCKCreateUDPClient1( HWND pHwnd, UINT uMesg, long *handle );

/***********************************************************************
 	int DLLMETHOD NWCKCreateUDPClient2( DWORD pThread, UINT uMesg,long *handle );

	Parameters:
		pThread		- A thread id that will receive notification of socket
					  events
		uMesg		- A WM_USER + XXX mesage that will be sent to pHwnd
		handle		- A pointer to a long.  On success this will contain an
					  unique handle.  You need to use this handle in all 
					  subsequent calls
	Returns:
		0	- Success 
			  ipaddr will have a single dotted ip address i.e. 223.223.223.1
		-1	- Winsock not initialized
		-2	- Winsock error ( call NWCKLastWinSockError() )
		-3	- Not used (ignore)
		-4	- Low Memory problem 
			  Do NOT call NWCKLastWinSockError() since the handle is invalid

	Notes:
		None.
***********************************************************************/
int DLLMETHOD NWCKCreateUDPClient2( DWORD pThread, UINT uMesg, long *handle );

/***********************************************************************
	int DLLMETHOD NWCKSetUDPBufferSize( long handle, UINT buffSize );

	Parameters:
		handle		- This should be the handle returned by an earlier call
					  to NWCKCreateUPDXXX()
		buffSize	- The size in bytes that this socket instance should
					  use when reading/writing data.  Remember Datagram 
					  sockets are not like Stream sockets.  If the buffer 
					  size is too small, you will loose what ever does 
					  not fit in the buffer.  An internal buffer of this
					  size is used for temporary storage.
				
	Returns:
		0	- Success 
		-1	- Winsock not initialized
		-2	- Out of memory ( call NWCKLastUDPWinSockError() )
		-3	- Not used (ignore)
		-4	- Invalid handle
			  

	Notes:
		You must call this function if you require a buffer bigger than
		1K (1024 bytes).  The default is 1K.  In most cases a 1K buffer is
		big enough.
***********************************************************************/
int DLLMETHOD NWCKSetUDPBufferSize( long handle, UINT buffSize );

/***********************************************************************
	int DLLMETHOD NWCKUDPWriteToPort( long handle, LPVOID data, int dataLen,
							LPSTR remoteName, int remotePort );

	Parameters:
		handle		- This should be the handle returned by an earlier call
					  to NWCKCreateUPDXXX()
		data		- A pointer to data that you wish to send.
		dataLen		- The size of the data pointed to by 'data'
		remoteName	- A NULL terminated string containing the remote machine 
					  name.
		remotePort	- The port number of the remote machine on which the UPD
					  server is bound

	Returns:
		0	- Success (means data has been queued successfully) you will be 
			  notified a little later when the data has been sent.  That is 
			  why the data pointer should be kept safe and valid
		-1	- Winsock not initialized
		-2	- Winsock error ( call NWCKLastUDPWinSockError() )
		-3	- Out of memory
		-4	- Invalid handle

	Notes:
		If your process is a UDP Server then use the 'NWCKUDPReply' function.
		Use this function only if your process is the UPD Client.

		The DLL will queue your data for sending.  When WINSOCK is ready
		to send it will send it.  You will receive a message with
		WPARAM set to the status and LPARAM set to the bytes sent.
***********************************************************************/
int DLLMETHOD NWCKUDPWriteToPort( long handle, LPVOID data, int dataLen,
							LPSTR remoteName, int remotePort );

/***********************************************************************
	int DLLMETHOD NWCKUDPWriteToService( long handle, LPVOID data, int dataLen,
								LPSTR remoteName, LPSTR remoteService );

	Parameters:
		handle		- This should be the handle returned by an earlier call
					  to NWCKCreateUPDXXX()
		data		- A pointer to data that you wish to send.
		dataLen		- The size of the data pointed to by 'data'
		remoteName	- A NULL terminated string containing the remote machine 
					  name.
		remoteService- The service name i.e. "ftp", "SMTP" etc.

	Returns:
		0	- Success (means data has been queued successfully) you will be 
			  notified a little later when the data has been sent.  That is 
			  why the data pointer should be kept safe and valid
		-1	- Winsock not initialized
		-2	- Winsock error ( call NWCKLastUDPWinSockError() )
		-3	- Out of memory
		-4	- Invalid handle

	Notes:
		If your process is a UDP Server then use the 'NWCKUDPReply' function.
		Use this function only if your process is the UPD Client.

		The DLL will queue your data for sending.  When WINSOCK is ready
		to send it will send it.  You will receive a message with
		WPARAM set to the status and LPARAM set to the bytes sent.
***********************************************************************/
int DLLMETHOD NWCKUDPWriteToService( long handle, LPVOID data, int dataLen,
							LPSTR remoteName, LPSTR remoteService );

/***********************************************************************
	int DLLMETHOD NWCKUDPRead( long handle, LPVOID data, int dataLen,
							unsigned char *replyAddr );

	Parameters:
		handle		- This should be the handle returned by an earlier call
					  to NWCKCreateUPDXXX()
		data		- A pointer to memory that you wish to receive the data 
					  into. 
		dataLen		- The size of the memory pointed to by 'data'
		replyAddr	- Should be a pointer to atleast 16Bytes of memory.  The
					  senders return address will be stored here.

	Returns:
		0	- Success 
		-1	- Winsock not initialized
		-2	- Winsock error ( call NWCKLastUDPWinSockError() )
		-3	- Not used (ignore)
		-4	- Invalid handle

	Notes:
		Never call this function until you receive a NWCK_UDP_RECVDATA 
		in the WPARAM.  The LPARAM will have the size in bytes of the data
		sent to you.  Allocate enough memory for LPARAM bytes.

		If your process is the UDP Server then the replyAddr buffer should
		be used when replying to the client.  If you are the server and 
		you wish to reply to the client then use the NWCKUDPReply function.
		It is similar to NWCKWriteToXXX()s.  It is especially designed for
		server response to clients.
***********************************************************************/
int DLLMETHOD NWCKUDPRead( long handle, LPVOID data, int dataLen, 
												unsigned char *replyAddr );

/***********************************************************************
	int DLLMETHOD NWCKUDPReply( long handle, LPVOID data, int dataLen,
												unsigned char *replyAddr );

	Parameters:
		handle		- This should be the handle returned by an earlier call
					  to NWCKCreateUPDXXX()
		data		- A pointer to memory contains the data you wish to send
					  back to the client in response.
		dataLen		- The size of the memory pointed to by 'data'
		replyAddr	- Is the data that was returned to you when you called
					  NWCKUDPRead().

	Returns:
		0	- Success (means data has been queued successfully) you will be 
			  notified a little later when the data has been sent.
		-1	- Winsock not initialized
		-2	- Winsock error ( call NWCKLastUDPWinSockError() )
		-3	- Out of memory
		-4	- Invalid handle
		-5	- This is not a server

	Notes:
		Use this function only if your process is a UDP Server.  This 
		function will send back data response to the UDP client.  It will
		use the 'replyAddr' data as the destination for your reply.
		The DLL will queue your data for sending.  When WINSOCK is ready
		to send it will send it.  You will receive a message with
		WPARAM set to the status and LPARAM set to the bytes sent.
***********************************************************************/
int DLLMETHOD NWCKUDPReply( long handle, LPVOID data, int dataLen,
												unsigned char *replyAddr );

/***********************************************************************
	int DLLMETHOD NWCKLastUDPWinSockError( long handle, int *err, 
														char *errStr );

	Parameters:
		handle		- This should be the handle returned by an earlier call
					  to NWCKCreateUPDXXX()
		err		- A pointer to an 'int' for error #
		errStr	- A pointer to an array of minimum 100 bytes.  Error message

	Returns:
		0	- Success 
			  err will have an error #
			  errStr will have an error message suitable for display
		-1	- Winsock not initialized
		-2	- Not used (ignore)
		-3	- Not used (ignore)
		-4	- Invalid handle

	Notes:
		There 3 versions of 'NWCKLastWinSockError'.  This one should
		be used only for datagram sockets.  Just make sure that you pass
		the correct 'handle'.  This way you can get errors on a per socket
		basis.
***********************************************************************/
int DLLMETHOD NWCKLastUDPWinSockError( long handle, int *err, 
														char *errStr );

/***********************************************************************
	int DLLMETHOD NWCKCloseUPD( long handle );

	Parameters:
		handle		- This should be the handle returned by an earlier call
					  to NWCKCreateUPDXXX()

	Returns:
		0	- Success (means socket is closed and memory freed)
		-1	- Winsock not initialized
		-2	- Not used (ignore)
		-3	- Not used (ignore)
		-4	- Invalid handle

	Notes:
		You must remember to call this function when you are done using a
		DATAGRAM socket.
***********************************************************************/
int DLLMETHOD NWCKCloseUDP( long handle );


/***********************************************************************
						STREAM SOCKET API

Stream sockets are more common because of their security features. Stream
sockets guarantee that data will be sent in a serialized fashion.  There
is no danger of packet#2 reaching before packet#1.  The basic principle 
behind stream sockets is the same as Datagrams with a twist.  The stream
orientated sockets always has a server on one side and 1-n clients across
the network.  The server always uses a primary socket that is used to 
"listen" for client connections.  When there is a client that wishes to 
connect, the server will accept the client connection.  In doing so winsock
creates a new socket for the server.  This new socket is then used for 
servicing that client connection.  Think of this new socket as a dedicated 
line from the server to a particular client.  What does the original socket
do?  It goes back to the task of "listening" for other clients that want to
connect to it.  Many popular client/server implementations use sockets in
this fashion.

SERVER
As a server you will typically call one of the NWCKCreateStreamServerX() 
functions.  Next you wait till your thread or window receives the user defined
message you set up earlier. When you receive this message the WPARAM
parameter will have a value of NWCK_STREAM_ACCEPT.  At this stage you will
call one of the NWCKCreateStreamCommX() functions.  When you call the
NWCKCreateStreamCommX() function make sure you have specified the correct
window (HWND) or correct ThreadId.  This function allows your server 
application to use a new window/thread for each connection to a new client.
The handle created by this function becomes your new COMMUNICATIONS handle 
with this new client.  You still need to preserve the original handle that
"listens".  Now you call NWCKStreamAccept() passing it the handle value
created in NWCKCreateStreamCommX() and the original handle.

From now on any data passed back and forth to the client is done using the
new handle.  Next you wait till your thread or window receives the user defined
message you set up earlier. When you receive this message the WPARAM
parameter will have a value of NWCK_STREAM_RECVDATA.  The LPARAM will contain
the size of the data received in bytes.  Your next step would be to allocate
x bytes of memory and call NWCKStreamRead() function.  The WPARAM could also
be NWCK_STREAM_READERROR.  This means some data was sent to us but there was an
error in receiving the data.  Call NWCKLastStreamWinSockError() to find out what 
occurred.  Now your Server application needs to know what information the 
Client could possibly send i.e. a C structure a query etc.  The data will 
come in bits and pieces.  You need to assemble them together on your side 
till you know that you have your expected data.

If you wish to send back a response or data you MUST use the 
NWCKStreamWrite() function.  If the function is successful you can free the 
memory associated with the data since the DLL makes a copy it.  After this 
call you must wait for a message sent back to your Window/Thread.  The 
message id will be the same as you set earlier.  The WPARAM will contain 
the value NWCK_STREAM_DATASENT.  The LPARAM is not used.  The WPARAM could 
also be NWCK_STREAM_WRITEERROR, this means there was an error.  You must then 
call NWCKLastStreamWinSockError().  TIP : With a stream socket you can send
as much data as you want i.e. a whole file.  The best thing to do is break
the data into chunks of 1K or 4K etc.  Then write to the stream, if the data
is sent without error then send the next packet and so on!

Now remember the whole process can happen again when a new client connects 
to your application server.  What you need to decide is how many clients 
your application will support.  A typical example is a Database Server.  
Some can handle 10 clients and some can handle 250.  It all depends on what
your application does.  When you receive a NWCK_STREAM_CONNECT you need to
know how many existing clients are being handled.  Then decide whether you 
wish to connect to the client or not.


CLIENT
As a client you need to know who you are going to communicate with.  You need
to know the Servers machine name and the Port # of the service or the 
service name itself.

First you would call one of the NWCKCreateStreamClientX() functions.  If the
handle is valid, you can connect to a Server.  Now you must call 
NWCKStreamConnectXX() function.  After this function you wait again till you 
receive your user defined message, with WPARAM set to NWCK_STREAM_CONNECT.  This
means you are connected.  Now you can start sending data.  To send data you must
call the NWCKStreamWrite() function.  If the function is successful you can free
the memory associated with your data since the DLL makes a copy of it.  After 
this function you must wait for a message sent to your Window/Thread.  The 
message id will be what you set up earlier.  The WPARAM will have a value of
NWCK_STREAM_DATASENT or NWCK_STREAM_WRITEERROR.

If the Server sent you a response/data, you will receive your user 
defined message to your Window/Thread.  When you receive this message the 
WPARAM parameter will have a value of NWCK_STREAM_RECVDATA.  The LPARAM will 
contain the size of the data received in bytes.  Your next step would be to 
allocate x bytes of memory and call NWCKStreamRead() function.  Now your
client application needs to know what information the Server could possibly
return i.e. a C structure a query result etc.  The data will come in bits 
and pieces.  You need to assemble them together on your side till you know
that you have your expected answer.

If you need to send data again to the Server, you will need to go through the
above steps again like you did when you first sent data.

MESSAGES
All messages will be sent to your Window in the form of PostMessage API.  
You can also receive messages to your thread in the form of 
PostThreadMessage API.  The "long handle" parameter should be preserved by 
you, it is used later to uniquely identify which thread or window instance 
owns the socket.

If during your winsock (stream) session the other side disconnects/shuts down
you will receive your user defined message with WPARAM set to 
NWCK_STREAM_SHUTDOWN.  At this stage you need to close your socket and decide
whether you want to start all over again or not.

When you are done using a Stream socket you MUST call NWCKCloseStream().  
This will allow the DLL to properly close the socket and to free all memory
associated with that instance.

MULITHREADED
The DLL is thread safe and can handle multiple threads.  The DLL uses all 
the Asynchronous APIs for Winsock.  So you don't really need to have a 
separate thread to handle Winsock message, unless you expect multiple
client connections.  Your application should be quite responsive using just 
the main thread.  Just design your application well first before coding.
***********************************************************************/



/***********************************************************************
	int DLLMETHOD NWCKCreateStreamServerWindow( int localPort, HWND pHwnd, 
												UINT uMesg, long *handle );

	Parameters:
		localPort	- An unique port #.  This will be used to bind the socket
		pHwnd		- A window handle that will receive notification of socket
					  events.
		uMesg		- A WM_USER + XXX mesage that will be sent to pHwnd
		handle		- A pointer to a long.  On success this will contain an
					  unique handle.  You need to use this handle in all 
					  subsequent calls
	Returns:
		0	- Success 
		-1	- Winsock not initialized
		-2	- Winsock error ( call NWCKLastWinSockError() )
		-3	- Bad 'localPort' #
		-4	- Low Memory problem 
			  Do NOT call NWCKLastWinSockError() since the handle is invalid

	Notes:
		Use this function to create a stream socket Server.  This version of the
		socket will send uMesg to your window.
***********************************************************************/
int DLLMETHOD NWCKCreateStreamServerWindow( int localPort, HWND pHwnd, 
													UINT uMesg, long *handle );

/***********************************************************************
	int DLLMETHOD NWCKCreateStreamServerWindow( int localPort, UINT threadId, 
												UINT uMesg, long *handle );

	Parameters:
		localPort	- An unique port #.  This will be used to bind the socket
		threadId	- A threadid that will receive notification of socket
					  events.
		uMesg		- A WM_USER + XXX mesage that will be sent to pHwnd
		handle		- A pointer to a long.  On success this will contain an
					  unique handle.  You need to use this handle in all 
					  subsequent calls
	Returns:
		0	- Success 
		-1	- Winsock not initialized
		-2	- Winsock error ( call NWCKLastWinSockError() )
		-3	- Bad 'localPort' #
		-4	- Low Memory problem 
			  Do NOT call NWCKLastWinSockError() since the handle is invalid

	Notes:
		Use this function to create a stream socket Server.  This version of the
		socket will send uMesg to your thread.
***********************************************************************/
int DLLMETHOD NWCKCreateStreamServerThread( int localPort, UINT threadId,
													UINT uMesg, long *handle );

/***********************************************************************
	int DLLMETHOD NWCKCreateStreamCommWindow( HWND pHwnd, UINT uMesg, long *handle );

	Parameters:
		pHwnd		- A window handle that will receive notification of socket
					  events.
		uMesg		- A WM_USER + XXX mesage that will be sent to pHwnd
		handle		- A pointer to a long.  On success this will contain an
					  unique handle.  You need to use this handle in all 
					  subsequent calls
	Returns:
		0	- Success 
		-1	- Winsock not initialized
		-2	- Winsock error ( call NWCKLastWinSockError() )
		-3	- Bad 'localPort' #
		-4	- Low Memory problem 
			  Do NOT call NWCKLastWinSockError() since the handle is invalid

	Notes:
		Use this function to create another socket object.  This object will
		be used for all future communications with the connected client socket
		stream.  This version of the socket will send uMesg to your window.
***********************************************************************/
int DLLMETHOD NWCKCreateStreamCommWindow( HWND pHwnd, UINT uMesg, long *handle );

/***********************************************************************
	int DLLMETHOD NWCKCreateStreamCommThread( UINT threadId, UINT uMesg, long *handle );

	Parameters:
		threadId	- A threadid that will receive notification of socket
					  events.
		uMesg		- A WM_USER + XXX mesage that will be sent to pHwnd
		handle		- A pointer to a long.  On success this will contain an
					  unique handle.  You need to use this handle in all 
					  subsequent calls
	Returns:
		0	- Success 
		-1	- Winsock not initialized
		-2	- Winsock error ( call NWCKLastWinSockError() )
		-3	- Bad 'localPort' #
		-4	- Low Memory problem 
			  Do NOT call NWCKLastWinSockError() since the handle is invalid

	Notes:
		Use this function to create another socket object.  This object will
		be used for all future communications with the connected client socket
		stream.  This version of the socket will send uMesg to your thread.
***********************************************************************/
int DLLMETHOD NWCKCreateStreamCommThread( UINT threadId, UINT uMesg, long *handle );


/***********************************************************************
	int DLLMETHOD NWCKStreamAccept( long origHandle, long commHandle )

	Parameters:
		origHandle	- This should be the handle returned by an earlier call
					  to NWCKCreateStreamServerXX()
		commHandle	- This should be the handle returned by an earlier call
					  to NWCKCreateStreamCommXX().

	Returns:
		0	- Success
		-1	- Winsock not initialized
		-2	- Winsock error ( call NWCKStreamLastWinSockError() )
		-4	- Invalid handle(s)

	Notes:
		Use this function to accept the client connection.  Make sure that you
		send the correct handle values in the correct parameter.  The first
		handle is the original socket object that listens for connections.
		The second handle value is the new object you created to converse
		with this particular client.
***********************************************************************/
int DLLMETHOD NWCKStreamAccept( long origHandle, long commHandle );

/***********************************************************************
	int DLLMETHOD NWCKCreateStreamClientWindow( HWND pHwnd, UINT uMesg, long *handle )

	Parameters:
		pHwnd		- A window handle that will receive notification of socket
					  events.
		uMesg		- A WM_USER + XXX mesage that will be sent to pHwnd
		handle		- A pointer to a long.  On success this will contain an
					  unique handle.  You need to use this handle in all 
					  subsequent calls
	Returns:
		0	- Success 
		-1	- Winsock not initialized
		-2	- Winsock error ( call NWCKLastWinSockError() )
		-4	- Low Memory problem 
			  Do NOT call NWCKLastWinSockError() since the handle is invalid

	Notes:
		Use this function to create a client socket object.  You will need to 
		connect to a Server before you can use this socket object.  This version
		of the socket will send uMesg to your window.
***********************************************************************/
int DLLMETHOD NWCKCreateStreamClientWindow( HWND pHwnd, UINT uMesg, long *handle );

/***********************************************************************
	int DLLMETHOD NWCKCreateStreamClientThread( UINT threadId, UINT uMesg, long *handle )

	Parameters:
		threadId	- A threadid that will receive notification of socket
					  events.
		uMesg		- A WM_USER + XXX mesage that will be sent to pHwnd
		handle		- A pointer to a long.  On success this will contain an
					  unique handle.  You need to use this handle in all 
					  subsequent calls
	Returns:
		0	- Success 
		-1	- Winsock not initialized
		-2	- Winsock error ( call NWCKLastWinSockError() )
		-4	- Low Memory problem 
			  Do NOT call NWCKLastWinSockError() since the handle is invalid

	Notes:
		Use this function to create a client socket object.  You will need to 
		connect to a Server before you can use this socket object.  This version
		of the socket will send uMesg to your thread.
***********************************************************************/
int DLLMETHOD NWCKCreateStreamClientThread( UINT threadId, UINT uMesg, long *handle );

/***********************************************************************
	int DLLMETHOD NWCKStreamConnectToPort( long handle, LPSTR remoteName, int remotePort )

	Parameters:
		handle		- This should be the handle returned by an earlier call
					  to NWCKCreateUPDXXX()
		remoteName	- A NULL terminated string containing the remote machine 
					  name.
		remotePort	- The port number of the remote machine on which the Stream
					  server socket is bound

	Returns:
		0	- Success
		-1	- Winsock not initialized
		-2	- Winsock error ( call NWCKStreamLastWinSockError() )
		-4	- Invalid handle

	Notes:
		Once you have create a Client Stream Socket, you will use this function
		to connect to a known port and a known server name.  Once you succeed
		here, you can start using this stream socket.
***********************************************************************/
int DLLMETHOD NWCKStreamConnectToPort( long handle, LPSTR remoteName, int remotePort );

/***********************************************************************
	int DLLMETHOD NWCKStreamConnectToService( long handle, LPSTR remoteName, 
														LPSTR remoteService )

	Parameters:
		handle		- This should be the handle returned by an earlier call
					  to NWCKCreateUPDXXX()
		remoteName	- A NULL terminated string containing the remote machine 
					  name.
		remoteService- A NULL terminated string with the name of the service you
					   wish to connect to, i.e. "ftp" etc.

	Returns:
		0	- Success
		-1	- Winsock not initialized
		-2	- Winsock error ( call NWCKStreamLastWinSockError() )
		-4	- Invalid handle

	Notes:
		Once you have create a Client Stream Socket, you will use this function
		to connect to a known service and a known server name.  Once you succeed
		here, you can start using this stream socket.
***********************************************************************/
int DLLMETHOD NWCKStreamConnectToService( long handle, LPSTR remoteName, LPSTR remoteService );

/***********************************************************************
	int DLLMETHOD NWCKStreamWrite( long handle, LPVOID data, int dataLen )

	Parameters:
		handle		- This should be the handle used to communicate.  For a 
					  server socket it will be the handle returned by
					  NWCKCreateStreamCommXXX().  For a Client socket it will
					  be the handle returned by the NWCKCreateStreamClientXX().
		data		- A pointer to the data you wish to send down the stream
		dataLen		- The size in bytes of the data pointed to by data

	Returns:
		0	- Success
		-1	- Winsock not initialized
		-2	- Winsock error ( call NWCKStreamLastWinSockError() )
		-4	- Invalid handle

	Notes:
		Use this function to write to the stream.  Make sure that the 
		NWCKSetStreamBufferSize() is used if you intend to use data buffers
		bigger than 1K.
***********************************************************************/
int DLLMETHOD NWCKStreamWrite( long handle, LPVOID data, int dataLen );

/***********************************************************************
	int DLLMETHOD NWCKStreamRead( long handle, LPVOID data, int dataLen )

	Parameters:
		handle		- This should be the handle used to communicate.  For a 
					  server socket it will be the handle returned by
					  NWCKCreateStreamCommXXX().  For a Client socket it will
					  be the handle returned by the NWCKCreateStreamClientXX().
		data		- A pointer to the memory where the data read is stored
		dataLen		- The size in bytes of memory pointed to by data

	Returns:
		0	- Success
		-1	- Winsock not initialized
		-2	- Winsock error ( call NWCKStreamLastWinSockError() )
		-4	- Invalid handle

	Notes:
		Use this function to write to the stream.  Make sure that the 
		NWCKSetStreamBufferSize() is used if you intend to use data buffers
		bigger than 1K.
***********************************************************************/
int DLLMETHOD NWCKStreamRead( long handle, LPVOID data, int dataLen );

/***********************************************************************
	int DLLMETHOD NWCKSetStreamBufferSize( long handle, UINT buffSize );

	Parameters:
		handle		- This should be the handle used to communicate.  For a 
					  server socket it will be the handle returned by
					  NWCKCreateStreamCommXXX().  For a Client socket it will
					  be the handle returned by the NWCKCreateStreamClientXX().
		buffSize	- The size in bytes that this socket instance should
					  use when reading/writing data.  An internal buffer of this
					  size is used for temporary storage.
				
	Returns:
		0	- Success 
		-1	- Winsock not initialized
		-2	- Out of memory ( call NWCKStreamLastWinSockError() )
		-3	- Not used (ignore)
		-4	- Invalid handle

	Notes:
		You must call this function if you require a buffer bigger than
		1K (1024 bytes).  The default is 1K.  In most cases a 1K buffer is
		big enough.
***********************************************************************/
int DLLMETHOD NWCKSetStreamBufferSize( long handle, UINT buffSize );

/***********************************************************************
	int DLLMETHOD NWCKLastStreamWinSockError( long handle, int *err, 
														char *errStr );

	Parameters:
		handle	- This should be the handle used to communicate.  For a 
				  server socket it will be the handle returned by
				  NWCKCreateStreamCommXXX().  For a Client socket it will
				  be the handle returned by the NWCKCreateStreamClientXX().
		err		- A pointer to an 'int' for error #
		errStr	- A pointer to an array of minimum 100 bytes.  Error message

	Returns:
		0	- Success 
			  err will have an error #
			  errStr will have an error message suitable for display
		-1	- Winsock not initialized
		-2	- Not used (ignore)
		-3	- Not used (ignore)
		-4	- Invalid handle

	Notes:
		There 3 versions of 'NWCKLastWinSockError'.  This one should
		be used only for datagram sockets.  Just make sure that you pass
		the correct 'handle'.  This way you can get errors on a per socket
		basis.
***********************************************************************/
int DLLMETHOD NWCKLastStreamWinSockError( long handle, int *err, 
														char *errStr );

/***********************************************************************
	int DLLMETHOD NWCKCloseStream( long handle );

	Parameters:
		handle		- This should be a valid stream socket object.

	Returns:
		0	- Success (means socket is closed and memory freed)
		-1	- Winsock not initialized
		-2	- Not used (ignore)
		-3	- Not used (ignore)
		-4	- Invalid handle

	Notes:
		You must remember to call this function when you are done using a
		STREAM socket.
***********************************************************************/
int DLLMETHOD NWCKCloseStream( long handle );



/**************** I N T E R N E T  I N T E R F A C E ******************

					NOTES ON USING INTERNET TOOLS
					*****************************

In the beginning... Well lets start with the creation of messages.  Your
User Interface will allow the user to enter a message to send and maybe a 
few file attachments.  Now you need to send this message and file to the 
internet, so you need to use the SMTP API.  To use the SMTP API you must 
already have a file with all messages and file attachments in it.  This file 
format is special to our implementation.  This is where the MIME API comes 
in.  You will use the MIME API to create this special file.  The MIME API 
will create a file that can be sent through the SMTP server.  Once the file 
is created with all the messages and file attachments, you will use the SMTP
API to send this file.  You probably should start a thread to do that.  When
the file is sent, your thread/window will be informed of success/failure.  
By using a separate thread your application can continue doing other things.

Next comes retrieval of messages from the internet.  POP(Post Office Protocol)
was designed to get mail off line.  We support the latest version 3.0 of POP.
First you have the POP3 module/API fetch the mail for you.  It will return 
success/failure.  It will also let you know how many messages came along.  
Now you use the MIME API again to convert the pop3 file into a readable 
format.  Then let MIME do its job.  It will extract each message and pass 
the text to you with other information in a structure.  You will store the
data in the structure if you wish.  Then you will see if there were any file
attachments.  If they were, then you will have the user recommend a new name
and destination.

The MIME API uses certain structures to assemble and disassemble this 
special file used by POP3 and SMTP API.  There are some structures you have
to use when communicating with the MIME API.

LIMITATION
None.

REQUIREMENTS
1. You must make sure that you have already started Winsock services.  This
   can be done by calling NWCKStartWinSock() at the start of your program.
   The Internet modules will NOT start/end winsock services.  That is 
   because it does not know if you are using winsock services in other parts
   of your application.
2. Our implementation will only handle upto 8 files as attachments.
***********************************************************************/

// Messages that could be sent to you in the WPARAM when you receive
// a user defined message to your thread or window.
#define	SMTP_SHUTDOWN		0		// Just informs you that mail services have ended
#define	SMTP_MEMORY_ERR		1		// Low memory problems
#define	SMTP_READ_ERR		2		// Winsock read error
#define	SMTP_WRITE_ERR		3		// Winsock write error
#define	SMTP_NOCONN_ERR		4		// Not able to connect to SMTP server
#define	SMTP_RESPONSE_ERR	5		// SMTP returned an ERROR as a response
#define	SMTP_FILEOPEN_ERR	6		// General File I/O unable to open file
#define	SMTP_FILEREAD_ERR	7		// General File I/O read error
#define	SMTP_NOMSG_ERR		8		// There were no messages to send
#define	SMTP_WINCREAT_ERR	9		// Unable to create hidden window
#define SMTP_NOSERVNAM_ERR	10		// Not a valid SMTP Server Name
#define	SMTP_SOCKCREAT_ERR	11		// Unable to create a stream socket
#define	SMTP_SERVDISCON_ERR	12		// Server disconnected us
#define	SMTP_NOSENDER_ERR	13		// Sender's E-Mail address not in file
#define SMTP_FILEFORMAT_ERR	14		// The send file(MIME created) format is bad 
#define	SMTP_MAILSENT_NUM	15		// The LPARAM is the message # sent OK.
#define	SMTP_MAILRCPT_ERR	16		// There was a bad Recipient Name in a message.
									// The LPARAM will have the message#.
#define	SMTP_CONNECT_OK		17		// Connected to SMTP Server - OK!
#define SMTP_MAILSENT_OK	18		// All Mail was sent successfully


/***********************************************************************
					Simple Mail Transfer Protocol
								API

To use this API set you need to follow these steps:

- Call 
	NWCKSMTPStartWindow( HWND myWindow, UINT myMesg, char *sendFile, long *handle );
	or
  	NWCKSMTPStartThread( UINT myThread, UINT myMesg, char *sendFile, long *handle );
- If it returns '0' then you can start using SMTP services
- Call NWCKSMTPSendMail();
- If it returns 0 then the mail transfer has started
- If it fails call NWCKSMTPClose( handle );

Once the service has started watch out for messages being sent to your
window/thread.  The message sent will be what you set in "myMesg".  The
WPARAM will contain 1 of the above SMTP_XXX values.

If there is any errors the SMTP module ends its connection with the Server.
You will first receive a SMTP_XXX_ERR in the WPARAM then a SMTP_SHUTDOWN 
value in the WPARAM.  When the WPARAM is set to SMTP_SHUTDOWN, your next
step should always be to call NWCKSMTPClose( handle );

If all the mail is sent successfully, then the WPARAM will first contain a
SMTP_MAILSENT_OK value then it will contain a SMTP_SHUTDOWN value.  The
value SMTP_MAILSENT_OK means everything was sent ok.

Well what are you waiting for???
***********************************************************************/


/***********************************************************************
	int DLLMETHOD NWCKSMTPStartWindow( HWND myWindow, UINT myMesg, 
											char *sendFile, long *handle );
	Parameters:
		myWindow	- A valid window handle that will receive messages
		myMesg		- A user defined (WM_USER+xx) message that will be sent
		sendFile	- Contains a valid file name to be used for sending data
		handle		- A pointer to a long, will store instance handle

	Returns:
		0	- Success
		-1	- Low memory unable to create SMTP module instance

	Notes:
		This is the first function you should call before you start using
		SMTP services.  This function should be called if you intend
		to use a window to receive status messages
***********************************************************************/
int DLLMETHOD NWCKSMTPStartWindow( HWND myWindow, UINT myMesg, 
											char *sendFile, long *handle );

/***********************************************************************
	int DLLMETHOD NWCKSMTPStartThread( UINT myThread, UINT myMesg, 
											char *sendFile, long *handle );
	Parameters:
		myThread	- A valid thread id that will receive messages
		myMesg		- A user defined (WM_USER+xx) message that will be sent
		sendFile	- Contains a valid file name to be used for sending data
		handle		- A pointer to a long, will store instance handle

	Returns:
		0	- Success
		-1	- Low memory unable to create SMTP module instance

	Notes:
		This is the first function you should call before you start using
		SMTP services.  This function should be called if you intend
		to use a thread to receive status messages
***********************************************************************/
int DLLMETHOD NWCKSMTPStartThread( UINT myThread, UINT myMesg, 
											char *sendFile, long *handle );

/***********************************************************************
	int DLLMETHOD NWCKSMTPSendMail( long handle, char *smtpServerName );

	Parameters:
		handle		- This should be a valid SMTP handle.
		smtpServerName - Is a valid Server name i.e netrail.net or
						 something.else.com etc.
	Returns:
		0	- Success
		-1	- Not used (ignore)
		-2	- Error ( you will receive a message soon enough)
		-3	- Not used (ignore)
		-4	- Invalid handle

	Notes:
		Use this function once you have a valid SMTP instance handle.  This
		function will start the process of sending the data in the file
		whose name you specified earlier.
***********************************************************************/
int DLLMETHOD NWCKSMTPSendMail( long handle, char *smtpServerName );

/***********************************************************************
	int DLLMETHOD NWCKSMTPClose( long handle );

	Parameters:
		handle		- This should be a valid SMTP handle.

	Returns:
		0	- Success
		-1	- Not used (ignore)
		-2	- Not used (ignore)
		-3	- Not used (ignore)
		-4	- Invalid handle

	Notes:
		You must remember to call this function when you are done using the
		SMTP module
***********************************************************************/
int DLLMETHOD NWCKSMTPClose( long handle );



/***********************************************************************
					Multipurpose Internet Mail Extension [MIME]
								v1.0 API

Our MIME API come in two parts:

* Preparing MIME messages
* Deciphering a MIME message from POP3 input

Preparing MIME messages
-----------------------
Before you can use the SMTP API you need to create a MIME specific file.  
Creating this file is simple.  Your user interface needs to do the following:

USER INPUT
- Their e-mail address i.e. sapient@netrail.net
- A list of receipients i.e. myfriend@abc.com, mynextfriend@xyz.com
- A list of Cc receipients i.e. myboss@abc.com, mybossesboss@xyz.com
- A subject matter of maximum 127 bytes.
- A list of files to attach with the message.  Format: "c:\bxb\myfile.txt"
  the total length of the file name should not exceed 127 bytes.  Actually
  MIME was created in order to send files embedded in a SMTP message.

Now that you have all the above information you will start by following:

1. Call NWCKMimePrepStart() this function will start of the MIME object.
2. Call NWCKMimePrepNewMesg() this function informs the MIME object that you started a new message
3. Call NWCKMimePrepTO() 1 - n times till you have passed it all the To:'s
4. Call NWCKMimePrepCC() 1 -n times till you have passed it all the Cc:'s
5. Call NWCKMimePrepSubj() to pass the subject matter for this message
6. Call NWCKMimePrepFile() 1 -n times till you have passed it all the file attachments
7. Call NWCKMimePrepData() 1 - n times till you have passed it all the data for the message
8. Call NWCKMimePrepEndMesg() this function tells the MIME object that this message ends
9. Call NWCKMimePrepFinish() this closes the MIME object and the file is flushed

If you have multiple messages to send in a single session, then repeat steps
2 - 8 once for each mail message.  You will notice that this set of API does
not have a Thread/Window callback feature.  That is because these function are 
not meant to be called from multiple threads/process.  You must prepare your
MIME files one at a time. 

HOW IS IT DONE
--------------
The MIME object will create structures and variables to hold information about
each message and as each message is completed it will start building the output 
file.

The MIME object will use a temporary file to stuff the body text of the mail
message.  This allows you to call step 7 as many times as needed to pass all 
the body text.  The ideal buffer size would be 512-1024 bytes.  Every time
you call step 7, the object will append the data to a temporary file.  Due to
this feature you can never use a MIME object in more than 1 thread.  Preparing
a MIME message is done 1 at a time.  Typically you will first prepare a MIME
message then use the SMTP service to send the mail message on its way.  After
that you can use MIME services again.

The MIME object will prepare a file that is compatible with our SMTP module.
All file attachments are encoded using Base64 encoding/decoding.  This is the
format used by MIME v1.0.

WARNING
-------
You are responsible for the body of the message (step 7).  What do we mean?
Well SMTP does not allow a single line to exceed 1000 characters including
the CRLF characters.  So when you prepare your text body for the mail message,
make sure that no line is longer than 997 characters.  You will have to parse
the users input text and format it if needed.  If a line is longer than 997
bytes then break it by inserting a CRLF somewhere in it.



Deciphering a MIME message from POP3 input
------------------------------------------
This 2nd part to the Mime module does decoding and parsing of retrieved
messages.  Before you use this message you must use the POP3 module to get
your mail.  If there are any messages/mail you then use the MimeGetXXX 
functions.  The purpose of this module is to allow you to store the 
different parts of a message in a database etc.  Then you can allow the user
to manage their mail messages etc.  The module parses mail messages 1 at a 
time.  That means you will instruct it to get the 1st message and then on
you will tell it to get the next and so on.  Here are the general steps
involved.

1.	NWCKMimeScanStart() passing it the file name created by POP3.
2.	NWCKMimeScanNumMessages() will return the number of messages in the file
3.	NWCKMimeScanGetFirst() will retrieve the first message for you
4.	NWCKMimeScanGetNext() will retrieve the next message
5.	NWCKMimeScanClose() will close the module and optionally delete the file

Step 4 should be repeated till all the messages have been retrieved.

One of the main things done by step 3-4 are to parse the message and 
extract plain text. Then it looks for file attachments, and automatically
decodes them to a file whose name is unique.  When you receive the structure
with the result of a message you will see what the name of the file(s) is.
You can then move/rename them to where your user wants them.  It is best to
erase the input file when done.  This will conserve disk space.
***********************************************************************/


/***********************************************************************
	int DLLMETHOD NWCKMimePrepStart( char *outFile, char *from, long *handle )

	Parameters:
		outFile		- A file name to create the MIME message(s) in
		from		- Your senders e-mail address "sender@xbxg.com"
		handle		- A pointer to long to store instance handle

	Returns:
		0	- Success
		-1	- Error low memory
		-2	- Not used (ignore)
		-3	- Not used (ignore)
		-4	- Not used (ignore)

	Notes:
		Must call this function only once.  Then when you are all done
		you must call the "NWCKMimePrepFinish()" function.  Only after that
		can you use the output file with SMTP.
***********************************************************************/
int DLLMETHOD NWCKMimePrepStart( char *outFile, char *from, long *handle );

/***********************************************************************
	int DLLMETHOD NWCKMimePrepNewMesg( long handle )

	Parameters:
		handle		- This should be a valid MIME handle.

	Returns:
		0	- Success
		-1	- Not used (ignore)
		-2	- Not used (ignore)
		-3	- Not used (ignore)
		-4	- Invalid handle

	Notes:
		Must call this function at the start of every new message
***********************************************************************/
int DLLMETHOD NWCKMimePrepNewMesg( long handle );

/***********************************************************************
	int DLLMETHOD NWCKMimePrepTO( long handle, char *toName )

	Parameters:
		handle		- This should be a valid MIME handle.
		toName		- A TO receipient i.e. "mystudent@school.edu"

	Returns:
		0	- Success
		-1	- Not used (ignore)
		-2	- string too long
		-3	- Not used (ignore)
		-4	- Invalid handle

	Notes:
		Call this function to specify a TO name no longer than 79 chars
***********************************************************************/
int DLLMETHOD NWCKMimePrepTO( long handle, char *toName );

/***********************************************************************
	int DLLMETHOD NWCKMimePrepCC( long handle, char *ccName )

	Parameters:
		handle		- This should be a valid MIME handle.
		ccName		- A CC receipient i.e. "mystudent@school.edu"

	Returns:
		0	- Success
		-1	- Not used (ignore)
		-2	- string too long
		-3	- Not used (ignore)
		-4	- Invalid handle

	Notes:
		Call this function to specify a CC name no longer than 79 chars
***********************************************************************/
int DLLMETHOD NWCKMimePrepCC( long handle, char *ccName );

/***********************************************************************
	int DLLMETHOD NWCKMimePrepSubj( long handle, char *subject )

	Parameters:
		handle		- This should be a valid MIME handle.
		subject		- A subject matter for this mail message

	Returns:
		0	- Success
		-1	- Not used (ignore)
		-2	- String too long
		-3	- Not used (ignore)
		-4	- Invalid handle

	Notes:
		Call this function to specify a subject matter max. 127 chars
***********************************************************************/
int DLLMETHOD NWCKMimePrepSubj( long handle, char *subject );

/***********************************************************************
	int DLLMETHOD NWCKMimePrepFile( long handle, char *fileAttach )

	Parameters:
		handle		- This should be a valid MIME handle.
		fileAttach	- A file name to attach to this message

	Returns:
		0	- Success
		-1	- Not used (ignore)
		-2	- String too long
		-3	- Not used (ignore)
		-4	- Invalid handle

	Notes:
		Call this function only if you have files to attach. Max 127 chars
***********************************************************************/
int DLLMETHOD NWCKMimePrepFile( long handle, char *fileAttach );

/***********************************************************************
	int DLLMETHOD NWCKMimePrepData( long handle, LPVOID body, int dataLen  );

	Parameters:
		handle		- This should be a valid MIME handle.
		body		- A pointer to the message text
		dataLen		- The length of the data pointed to by body

	Returns:
		0	- Success
		-1	- Not used (ignore)
		-2	- Error writing data to temp file
		-3	- Not used (ignore)
		-4	- Invalid handle

	Notes:
		Call this function as many times as you wish to pass the message body
		text.  This way you can do it in small chunks and not be bothered with
		memory issues.
***********************************************************************/
int DLLMETHOD NWCKMimePrepData( long handle, LPVOID body, int dataLen  );

/***********************************************************************
	int DLLMETHOD NWCKMimePrepEndMesg( long handle );

	Parameters:
		handle		- This should be a valid MIME handle.

	Returns:
		0	- Success
		-1	- Not used (ignore)
		-2	- Error writing output
		-3	- Not used (ignore)
		-4	- Invalid handle

	Notes:
		You must remember to call this function when you are done with a given
		mail message
***********************************************************************/
int DLLMETHOD NWCKMimePrepEndMesg( long handle );

/***********************************************************************
	int DLLMETHOD NWCKMimePrepFinish( long handle );

	Parameters:
		handle		- This should be a valid MIME handle.

	Returns:
		0	- Success
		-1	- Not used (ignore)
		-2	- Not used (ignore)
		-3	- Not used (ignore)
		-4	- Invalid handle

	Notes:
		You must remember to call this function when you are done using the
		MIME module
***********************************************************************/
int DLLMETHOD NWCKMimePrepFinish( long handle );



// MIME PARSING API	- Used to parse/extract data from POP3 file
typedef struct {
	char			msgId[ 64 ];
	char			dateStr[ 40 ];
	char			fromStr[ 80 ];
	char			subjStr[ 256 ];
	char			*toStr[ 32 ];
	int				toNum;
	char			*ccStr[ 32 ];
	int				ccNum;
	char			*fileAttach[ 8 ];
	int				fileNum;
	unsigned char	*dataStr;
	int				dataSize;
} NWCKMIMEMSG, *LPNWCKMIMEMSG;

/*
						STRUCTURE MEMBERS
	msgId
		A string that contains a unique messages id.
	dateStr
		A string containing the date message was created/sent
	fromStr
		An Interney address of the sender
	subjStr
		A pointer to a string containing a subject matter.  If the subject
		matter string is longer than 255 characters, you will loose the 
		rest.
	toStr
		A 2 dimensional array of 32 To strings.  You name/address may be in 
		it.  You will get the first 32 To strings.  You will loose the rest.
	toNum
		Tells you the number of To strings in the 'toStr'.  That means there
		are 0 to (toNum-1) strings.
	ccStr
		A 2 dimensional array of 32 Cc strings.  You name/address may be in 
		it.  You will get the first 32 Cc strings.  You will loose the rest.
	ccNum
		Tells you the number of Cc strings in the 'ccStr'.  That means there
		are 0 to (ccNum-1) strings.
	fileAttach
		Is a 2 dimensional array of all the file names that were created
		for you from this message.  These files were sent as attachments
		and were decoded using Base64 algorithm.  You can rename/move them
		when you wish.  A total of 8 attached files is supported.
	fileNum
		Is an integer that stores the number of 'fileAttach' elements.  You
		need to use this value to determine the size of the array.
	dataStr
		This pointer points to the actual data as ascii characters.  This
		is the message sent to you.
	dataSize
		Is an integer that stores the size of 'dataStr' in bytes.  You 
		should rely on this number.  We will NULL terminate 'dataStr', but
		you can never tell what is in a file.

							NOTES
							-----
This structure is created for you by the MIME module.  You should treat
this structure pointer and all its members as READ-ONLY.  Do not mess with
the data.  Copy what you need.  When you are done the memory will be freed
by the MIME module.  The simple rule is : "If you did not allocate the 
memory, then don't mess with it.  The module responsible for allocating
the memory will free it."

How is this done?  When you call NWCKMimeScanGetFirst(), it will return a
pointer to this structure.  If you call NWCKMimeScanGetNext(), it will
free the memory and reallocate new memory.  When you finally call 
NWCKMimeScanClose(), it will free any memory that was not freed.
*/


/***********************************************************************
	int DLLMETHOD NWCKMimeScanStart( char *inFile, char *wDir, long *handle )

	Parameters:
		inFile		- The name of the file created by POP3
		wDir		- The name of the working directory.  This is where
					  temp files and the final file attachments are stored.
					  The length should not exceed 159 chars.  Make sure that
					  this directory exists, else the module will try to use
					  the tmp directory if any else it will fail.
		handle		- A pointer to long to store instance handle

	Returns:
		0	- Success
		-1	- Error low memory
		-2	- Not used (ignore)
		-3	- Not used (ignore)
		-4	- Not used (ignore)

	Notes:
		Must call this function only once.  Then when you are all done
		you must call the "NWCKMimeScanClose()" function.
***********************************************************************/
int DLLMETHOD NWCKMimeScanStart( char *inFile, char *wDir, long *handle );

/***********************************************************************
	int DLLMETHOD NWCKMimeScanNumMessages( long handle, int *nMsg )
	
	Parameters:
		handle		- A valid handle to a MimeScan module
		nMsg		- A pointer to int to store result

	Returns:
		0	- Success
		-1	- Error low memory
		-2	- Not used (ignore)
		-3	- Not used (ignore)
		-4	- Not used (ignore)

	Notes:
		Must call this function only once.  Then when you are all done
		you must call the "NWCKMimeScanClose()" function.
***********************************************************************/
int DLLMETHOD NWCKMimeScanNumMessages( long handle, int *nMsg );

/***********************************************************************
	LPNWCKMIMEMSG DLLMETHOD NWCKMimeScanGetFirst( long handle )
	
	Parameters:
		handle		- A valid handle to a MimeScan module

	Returns:
		On success it will return a valid pointer to a NWCKMIMEMSG
		structure.
		On failure it will return NULL.  A NULL return value could mean
		an invalid 'handle' value or low memory problems.

	Notes:
		Must call this function only once to retrieve the very first
		mail message.  You must call NWCKMimeScanGetNext() to get the 
		remaining mail messages.
***********************************************************************/
LPNWCKMIMEMSG DLLMETHOD NWCKMimeScanGetFirst( long handle );

/***********************************************************************
	LPNWCKMIMEMSG DLLMETHOD NWCKMimeScanGetNext( long handle )
	
	Parameters:
		handle		- A valid handle to a MimeScan module

	Returns:
		On success it will return a valid pointer to a NWCKMIMEMSG
		structure.
		On failure it will return NULL.  A NULL return value could mean
		an invalid 'handle' value, low memory problems or there is no more
		mail messages in this file.  Finally it could mean file read error.

	Notes:
		You must call this function to retrieve all mail messages except
		the very first.
***********************************************************************/
LPNWCKMIMEMSG DLLMETHOD NWCKMimeScanGetNext( long handle );

/***********************************************************************
	int DLLMETHOD NWCKMimeScanClose( long handle )

	Parameters:
		handle		- This should be a valid MIME handle.

	Returns:
		0	- Success
		-1	- Not used (ignore)
		-2	- Not used (ignore)
		-3	- Not used (ignore)
		-4	- Invalid handle

	Notes:
		You must remember to call this function when you are done using the
		MIME module
***********************************************************************/
int DLLMETHOD NWCKMimeScanClose( long handle );




/***********************************************************************
						Post Office Protocol (POP)
								v3.0 API

Using the POP3 services is very easy.  Its the opposite of SMTP.  The following
are the basic steps to POP3 mail retrieval.

1.	Call NWCKPOP3StartWindow() or NWCKPOP3StartThread().  This function 
	creates	an instance of the POP3 class.  It assigns your handle a unique
	id.
2.	IF YOU WANT TO PULL THE MAIL IN
		If you have a valid handle then call NWCKPOP3PullMail() with all the 
		parameters it needs.
	IF YOU JUST WANT TO QUERY THE POP3 SERVER
		Call NWCKPOP3IsThereMail().  This function goes through all the steps
		required to connect to the server etc.  It will stop after it has 
		received a response to the 'STAT' command.  At this stage it will send
		you a POP3_TOTALMAIL_MSG message with LPARAM set to the # of messages
		waiting.  Soon after that it will close the connection and send you a
		POP3_SHUTDOWN message.

	The NWCKPOP3PullMail and NWCKPOP3IsThereMail are mutually exclusive.  That
	means you can call one or the other not both at the same time.  If you want 
	to query first and then pull mail, you need to wait till you receive the 
	POP3_SHUTDOWN message after calling NWCKPOP3IsThereMail.  Then call 
	NWCKPOP3PullMail.

3.	Wait for messages.  Based on what message you get you might have to abort.
4.	Call NWCKPOP3NumMail() will return the number of mail messages pulled down
5.	Call NWCKPOP3Close().

Thats it!  The message that you will receive in your window/thread will have
the MessageId you setup, the WPARAM will have one of the POP3_XXX constants.
When you receive a POP3_SHUTDOWN, you MUST call NWCKPOP3Close().  All the
messages will be written out to the file you specified.  If you received mail
and all came through OK, then the WPARAM will be "POP3_MAILRTRV_OK".  Now you
can pass this file off to the NWCKMimePullXXX() functions.

WARNING
-------
At any given time a single mail box can be queried for mail by only 1 process.
That means the POP3 server will only handle 1 client for a given mail box at a
given time.  So DO NOT attempt to pull mail from your mail box in 2 threads
at the same time.  You can however use another thread to pull messages from
another mail box.
***********************************************************************/
// Messages that could be sent to you in the WPARAM when you receive
// a user defined message to your thread or window.
#define	POP3_SHUTDOWN		0		// Just informs you that pop services have ended
#define	POP3_MEMORY_ERR		1		// Low memory problems
#define	POP3_READ_ERR		2		// Winsock read error
#define	POP3_WRITE_ERR		3		// Winsock write error
#define	POP3_NOCONN_ERR		4		// Not able to connect to POP server
#define	POP3_RESPONSE_ERR	5		// POP returned an ERROR (-ERR) as a response
#define	POP3_FILEOPEN_ERR	6		// General File I/O unable to open file
#define	POP3_FILEREAD_ERR	7		// General File I/O read error
#define	POP3_FILEWRIT_ERR	8		// General File I/O write error
#define	POP3_NOMSG_ERR		9		// There were no messages to retrieve(not an error)
#define	POP3_WINCREAT_ERR	10		// Unable to create hidden window
#define POP3_NOSERVNAM_ERR	11		// Not a valid POP Server Name
#define	POP3_SOCKCREAT_ERR	12		// Unable to create a stream socket
#define	POP3_SERVDISCON_ERR	13		// Server disconnected us
#define	POP3_NOUSERNAM_ERR	14		// Not a valid USER name
#define POP3_NOPWORD_ERR	15		// Not a valid PASSWORD
#define POP3_RTRVNG_MSG		16		// Retrieved a message Ok.  Mesg# in LPARAM
#define POP3_CONNECT_OK		17		// We connected to POP server OK!
#define POP3_TOTALMAIL_MSG	18		// The LPARAM will have the total mail messages for you
#define POP3_MAILRTRV_OK	19		// All Mail was retrieved successfully


/***********************************************************************
	int DLLMETHOD NWCKPOP3StartWindow( HWND myWindow, UINT myMesg, 
											char *outFile, long *handle );
	Parameters:
		myWindow	- A valid window handle that will receive messages
		myMesg		- A user defined (WM_USER+xx) message that will be sent
		outFile		- Contains a valid file name to be used for storing mail
		handle		- A pointer to a long, will store instance handle

	Returns:
		0	- Success
		-1	- Low memory unable to create POP3 module instance

	Notes:
		This is the first function you should call before you start using
		POP3 services.  This function should be called if you intend
		to use a window to receive status messages
***********************************************************************/
int DLLMETHOD NWCKPOP3StartWindow( HWND myWindow, UINT myMesg, 
											char *outFile, long *handle );

/***********************************************************************
	int DLLMETHOD NWCKPOP3StartThread( UINT myThread, UINT myMesg, 
											char *outFile, long *handle )
	Parameters:
		myThread	- A valid thread Id that will receive messages
		myMesg		- A user defined (WM_USER+xx) message that will be sent
		outFile		- Contains a valid file name to be used for storing mail
		handle		- A pointer to a long, will store instance handle

	Returns:
		0	- Success
		-1	- Low memory unable to create POP3 module instance

	Notes:
		This is the first function you should call before you start using
		POP3 services.  This function should be called if you intend
		to use a thread to receive status messages
***********************************************************************/
int DLLMETHOD NWCKPOP3StartThread( UINT myThread, UINT myMesg, 
											char *outFile, long *handle );

/***********************************************************************
	int DLLMETHOD NWCKPOP3IsThereMail( long handle, char *popServer, 
											char *user, char *passWord )

	Parameters:
		handle		- A valid handle to a POP3 module
		popServer	- A valid POP3/Mail Server name i.e. netrail.net
		user		- The user who's mail box you wish to tap
		passWord	- A password to go with this user

	Returns:
		-1	- Not used (ignore)
		-2	- Error ( you will receive a message soon enough)
		-3	- Not used (ignore)
		-4	- Invalid handle

	Notes:
		This function is provided as a way to just query the POP3 server.  You
		can use this function to find out if there is mail and if so how many.
		This function does not pull the mail for you.  The mail is left on the
		server.
		If you decide to just query not not pull mail, you still need to call
		the NWCKPOP3Close() function.
***********************************************************************/
int DLLMETHOD NWCKPOP3IsThereMail( long handle, char *popServer, char *user, 
															char *passWord );

/***********************************************************************
	int DLLMETHOD NWCKPOP3PullMail( long handle, char *popServer, char *user, 
						char *passWord, BOOL delMsg  )
	Parameters:
		handle		- A valid handle to a POP3 module
		popServer	- A valid POP3/Mail Server name i.e. netrail.net
		user		- The user who's mail box you wish to tap
		passWord	- A password to go with this user
		delMsg		- TRUE = Delete messages after retrieval 
					  FALSE = Leave mail on server

	Returns:
		0	- Success
		-1	- Not used (ignore)
		-2	- Error ( you will receive a message soon enough)
		-3	- Not used (ignore)
		-4	- Invalid handle

	Notes:
		Call this function to start pulling mail from the POP3 server.  The
		user and password must be correct or else it will fail.
***********************************************************************/
int DLLMETHOD NWCKPOP3PullMail( long handle, char *popServer, char *user, 
											char *passWord, BOOL delMsg  );

/***********************************************************************
	int DLLMETHOD NWCKPOP3NumMail( long handle, int *numMsg );
	Parameters:
		handle		- A valid handle to a POP3 module
		numMsg		- A pointer to int that will store the result

	Returns:
		0	- Success
		-1	- Not used (ignore)
		-2	- Not used (ignore)
		-3	- Not used (ignore)
		-4	- Invalid handle

	Notes:
		Call this function to find out how many messages were retrieved.  If
		there are no messages for this user, you will receive a message with
		WPARAM = POP3_NOMSG_ERR.  Although it sounds like an error its not an
		error.
***********************************************************************/
int DLLMETHOD NWCKPOP3NumMail( long handle, int *numMsg );

/***********************************************************************
	int DLLMETHOD NWCKPOP3Close( long handle );

	Parameters:
		handle		- This should be a valid POP3 handle.

	Returns:
		0	- Success
		-1	- Not used (ignore)
		-2	- Not used (ignore)
		-3	- Not used (ignore)
		-4	- Invalid handle

	Notes:
		You must remember to call this function when you are done using the
		POP3 module
***********************************************************************/
int DLLMETHOD NWCKPOP3Close( long handle );


#ifdef __cplusplus
}
#endif


#endif
