#if !defined(__rchedtcn_h)              // Sentry, use file only if it's not already included.
#define __rchedtcn_h

/*  Project richctrl
    DHB Software
    Copyright  1996. All Rights Reserved.

    SUBSYSTEM:    richctrl.apx Application
    FILE:         rchedtcn.h
    AUTHOR:       David H. Borg


    OVERVIEW
    ========
    Class definition for RichEditControl (TEdit).      
*/

#include <owl\owlpch.h>
#pragma hdrstop


#include <commctrl.h>			// Windows 95 common controls
#include <richedit.h>			// Windows 95 rich text editor
#include "rchctrla.rh"            // Definition of all resources.


//{{TEdit = RichEditControl}}
class RichEditControl : public TEdit {
public:
    RichEditControl (TWindow* parent, int id, const char far* text, int x, int y, int w, int h, uint textLen = 0, bool multiline = false, TModule* module = 0);
    virtual ~RichEditControl ();

public:
    TFindReplaceDialog::TData FindReplaceData;
    TFindReplaceDialog*       FindReplaceDialog;
    uint                      FindReplaceCmd;

    void DoFind();

protected:
	TOpenSaveDialog::TData FileData;
	TChooseFontDialog::TData FontData;
	char far* FileName;
	CHARFORMAT charFormat;	// common character format structure
	PARAFORMAT paraFormat;	// common paragraph format structure
	int streamFormat;
	bool updtFontFace, updtFontSize;	// time to update character font info

	bool CanClear();
	void RichEditControl::SetFileName (const char far* newName);
	void DoFileOpenCmd();
    bool Open( const char far* fileName);
	static DWORD CALLBACK OpenCallback(DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG FAR *pcb);
	void DoSaveAsCmd();
    bool Save( const char far* fileName);
	static DWORD CALLBACK SaveCallback (DWORD dwCookie, LPBYTE pbBuff, LONG cb, LONG FAR *pcb);
	void UpdateFormats();

//{{RichEditControlVIRTUAL_BEGIN}}
public:
    virtual char far* GetClassName ();
    virtual void Clear ();
    virtual void SetupWindow ();
    virtual bool CanClose ();
    virtual void Paint (TDC& dc, bool erase, TRect& rect);
//{{RichEditControlVIRTUAL_END}}

//{{RichEditControlRSP_TBL_BEGIN}}
protected:
	LRESULT CmChangeFontFace(WPARAM wParam, LPARAM lParam);
	void CmEnFontFace (TCommandEnabler &tce);
	LRESULT CmChangeFontSize(WPARAM wParam, LPARAM lParam);
	void CmEnFontSize (TCommandEnabler &tce);
    void CmFileNew ();
    void CmFileSave ();
    void CmEnFileSave (TCommandEnabler &tce);
    void CmFileOpenRtf ();
    void CmFileOpenText ();
    void CmSaveAsRtf ();
    void CmSaveAsText ();
    void CmEnSaveAs (TCommandEnabler &tce);
    void CmEditFind ();
    void CmEnEditFindReplace (TCommandEnabler &tce);
    void CmEditFindNext ();
    void CmEnEditFindNext (TCommandEnabler &tce);
    LRESULT EvFindMsg(WPARAM, LPARAM);  // common dialog message
    void CmFormatBold ();
    void CmEnFormatBold (TCommandEnabler &tce);
    void CmFormatItalic ();
    void CmEnFormatItalic (TCommandEnabler &tce);
    void CmFormatUnderline ();
    void CmEnFormatUnderline (TCommandEnabler &tce);
    void CmFormatLeft ();
    void CmEnFormatLeft (TCommandEnabler &tce);
    void CmFormatCenter ();
    void CmEnFormatCenter (TCommandEnabler &tce);
    void CmFormatRight ();
    void CmEnFormatRight (TCommandEnabler &tce);
    void CmFormatFont ();
    void CmEnFormatFont (TCommandEnabler &tce);
    void EvKeyDown (uint key, uint repeatCount, uint flags);
    void EvLButtonDown (uint modKeys, TPoint& point);
    void CmFilePrint ();
//{{RichEditControlRSP_TBL_END}}
DECLARE_RESPONSE_TABLE(RichEditControl);
};    //{{RichEditControl}}


#endif                                      // __rchedtcn_h sentry.

